namespace SvgDrawing {
	export interface Rect {
		x: number;
		y: number;
		width: number;
		height: number;
	}
	
	export class Viewport {
		constructor(public logical: Rect, public view: Rect) {}
		
		xLogicalToView(x: number) { return (x - this.logical.x) / this.logical.width * this.view.width + this.view.x; }
		yLogicalToView(y: number) { return (y - this.logical.y) / this.logical.height * this.view.height + this.view.y; }
	}
	
	export class SvgElement {
		public viewport: Viewport;
		
		constructor(private htmlElement: HTMLElement) {
			let viewBox = htmlElement.getAttribute('viewBox').split(' ');
			let r: Rect = { x: Number(viewBox[0]), y: Number(viewBox[1]), width: Number(viewBox[2]), height: Number(viewBox[3]) };
			this.viewport = new Viewport(r, { x: r.x, y: r.y + r.height, width: r.width, height: -r.height });
		}
		
		graph(y: number[]): SVGPathElement;
		graph(x: number[], y: number[]): SVGPathElement;
		graph(arg1: number[], arg2?: number[]) {
			let x: number[] | null = arg1;
			let y: number[] = arg2;
			if(!y) {
				y = arg1;
				x = null;
			}
			
			let num = y.length;
			console.assert(!x || num == x.length);
			
			if(num <= 1) return null;
			
			let xStep = 6;
			
			let coordinates = 'M'+Math.round(this.viewport.xLogicalToView(x? x[0] : 0))+','+Math.round(this.viewport.yLogicalToView(y[0]));
			coordinates += ' L';
			let lineStartX = x ? x[0] : 0;
			let prevX = lineStartX;
			let prevY = y[0];
			let yDir = y[1] > y[0] ? 1 : -1;
			
			let count = 0;
			for(let idx = 0; idx < num; ++idx) {
				let isLast = (idx == num - 1);
				
				let newX = x ? x[idx] : idx;
				let newY = y[idx];
				let dir = isLast ? 0 : (y[idx+1] > newY ? 1 : -1);
				
				if(newX >= lineStartX + xStep || dir != yDir || isLast) {
					coordinates += Math.round(this.viewport.xLogicalToView(newX))+','+Math.round(this.viewport.yLogicalToView(newY));
					if(!isLast) coordinates += ' ';
					lineStartX = newX;
					yDir = isLast ? 0 : (y[idx+1] > newY ? 1 : -1);
					++count;
				}
				prevY = newY;
			}
			
			console.log(count);
			
			let path = document.createElementNS('http://www.w3.org/2000/svg','path');
			path.setAttribute('class','graph');
			path.setAttribute('d', coordinates);
			this.htmlElement.append(path);
			
			return path;
		}
	}
}
